@@ -75,12 +75,12 @@ HRESULT d3d_execute_buffer_execute(struct d3d_execute_buffer *buffer,
     for (;;)
     {
         D3DINSTRUCTION *current = (D3DINSTRUCTION *)instr;
-	BYTE size;
-	WORD count;
-	
-	count = current->wCount;
-	size = current->bSize;
-	instr += sizeof(D3DINSTRUCTION);
+        BYTE size;
+        WORD count;
+
+        count = current->wCount;
+        size = current->bSize;
+        instr += sizeof(*current);
         primitive_size = 0;
 
         switch (current->bOpcode)
@@ -88,7 +88,7 @@ HRESULT d3d_execute_buffer_execute(struct d3d_execute_buffer *buffer,
             case D3DOP_POINT:
             {
                 const D3DPOINT *p = (D3DPOINT *)instr;
-                wined3d_device_set_primitive_type(device->wined3d_device, WINED3D_PT_POINTLIST);
+                wined3d_device_set_primitive_type(device->wined3d_device, WINED3D_PT_POINTLIST, 0);
                 wined3d_device_set_stream_source(device->wined3d_device, 0,
                         buffer->dst_vertex_buffer, 0, sizeof(D3DTLVERTEX));
                 wined3d_device_set_vertex_declaration(device->wined3d_device,
@@ -103,7 +103,7 @@ HRESULT d3d_execute_buffer_execute(struct d3d_execute_buffer *buffer,
 
             case D3DOP_LINE:
                 primitive_size = 2;
-                wined3d_device_set_primitive_type(device->wined3d_device, WINED3D_PT_LINELIST);
+                wined3d_device_set_primitive_type(device->wined3d_device, WINED3D_PT_LINELIST, 0);
                 /* Drop through. */
             case D3DOP_TRIANGLE:
             {
@@ -116,7 +116,7 @@ HRESULT d3d_execute_buffer_execute(struct d3d_execute_buffer *buffer,
 
                 if (!primitive_size)
                 {
-                    wined3d_device_set_primitive_type(device->wined3d_device, WINED3D_PT_TRIANGLELIST);
+                    wined3d_device_set_primitive_type(device->wined3d_device, WINED3D_PT_TRIANGLELIST, 0);
                     primitive_size = 3;
                 }
 
@@ -124,13 +124,19 @@ HRESULT d3d_execute_buffer_execute(struct d3d_execute_buffer *buffer,
 
                 if (buffer->index_size < index_count)
                 {
-                    struct wined3d_buffer *new_buffer;
                     unsigned int new_size = max(buffer->index_size * 2, index_count);
+                    struct wined3d_buffer *new_buffer;
+                    struct wined3d_buffer_desc desc;
 
-                    hr = wined3d_buffer_create_ib(device->wined3d_device, new_size * sizeof(*indices),
-                            WINED3DUSAGE_DYNAMIC | WINED3DUSAGE_WRITEONLY, WINED3D_POOL_DEFAULT,
-                            NULL, &ddraw_null_wined3d_parent_ops, &new_buffer);
-                    if (FAILED(hr))
+                    desc.byte_width = new_size * sizeof(*indices);
+                    desc.usage = WINED3DUSAGE_DYNAMIC | WINED3DUSAGE_WRITEONLY | WINED3DUSAGE_STATICDECL;
+                    desc.bind_flags = WINED3D_BIND_INDEX_BUFFER;
+                    desc.access = WINED3D_RESOURCE_ACCESS_GPU | WINED3D_RESOURCE_ACCESS_MAP;
+                    desc.misc_flags = 0;
+                    desc.structure_byte_stride = 0;
+
+                    if (FAILED(hr = wined3d_buffer_create(device->wined3d_device, &desc,
+                            NULL, NULL, &ddraw_null_wined3d_parent_ops, &new_buffer)))
                         return hr;
 
                     buffer->index_size = new_size;
@@ -203,10 +209,10 @@ HRESULT d3d_execute_buffer_execute(struct d3d_execute_buffer *buffer,
                 break;
             }
 
-	    case D3DOP_MATRIXLOAD:
-	        WARN("MATRIXLOAD-s     (%d)\n", count);
-	        instr += count * size;
-	        break;
+            case D3DOP_MATRIXLOAD:
+                WARN("MATRIXLOAD-s     (%u)\n", count);
+                instr += count * size;
+                break;
 
             case D3DOP_MATRIXMULTIPLY:
                 TRACE("MATRIXMULTIPLY   (%d)\n", count);
@@ -351,19 +357,37 @@ HRESULT d3d_execute_buffer_execute(struct d3d_execute_buffer *buffer,
                 }
                 break;
 
-	    case D3DOP_TEXTURELOAD: {
-	        WARN("TEXTURELOAD-s    (%d)\n", count);
+            case D3DOP_TEXTURELOAD:
+                TRACE("TEXTURELOAD    (%u)\n", count);
 
-		instr += count * size;
-	    } break;
+                for (i = 0; i < count; ++i)
+                {
+                    D3DTEXTURELOAD *ci = (D3DTEXTURELOAD *)instr;
+                    struct ddraw_surface *dst, *src;
 
-	    case D3DOP_EXIT: {
-	        TRACE("EXIT             (%d)\n", count);
-		/* We did this instruction */
-		instr += size;
-		/* Exit this loop */
-		goto end_of_buffer;
-	    } break;
+                    instr += size;
+
+                    if (!(dst = ddraw_get_object(&device->handle_table,
+                            ci->hDestTexture - 1, DDRAW_HANDLE_SURFACE)))
+                    {
+                        WARN("Invalid destination texture handle %#x.\n", ci->hDestTexture);
+                        continue;
+                    }
+                    if (!(src = ddraw_get_object(&device->handle_table,
+                            ci->hSrcTexture - 1, DDRAW_HANDLE_SURFACE)))
+                    {
+                        WARN("Invalid source texture handle %#x.\n", ci->hSrcTexture);
+                        continue;
+                    }
+
+                    IDirect3DTexture2_Load(&dst->IDirect3DTexture2_iface, &src->IDirect3DTexture2_iface);
+                }
+                break;
+
+            case D3DOP_EXIT:
+                TRACE("EXIT             (%u)\n", count);
+                instr += size;
+                goto end_of_buffer;
 
             case D3DOP_BRANCHFORWARD:
                 TRACE("BRANCHFORWARD    (%d)\n", count);
@@ -380,26 +404,28 @@ HRESULT d3d_execute_buffer_execute(struct d3d_execute_buffer *buffer,
                                 instr = (char*)current + ci->dwOffset;
                                 break;
                             }
-			}
-		    } else {
-		        if (ci->bNegate) {
+                        }
+                    }
+                    else
+                    {
+                        if (ci->bNegate)
+                        {
                             TRACE(" Branch to %d\n", ci->dwOffset);
                             if (ci->dwOffset) {
                                 instr = (char*)current + ci->dwOffset;
                                 break;
                             }
-			}
-		    }
+                        }
+                    }
 
-		    instr += size;
+                    instr += size;
                 }
                 break;
 
-	    case D3DOP_SPAN: {
-	        WARN("SPAN-s           (%d)\n", count);
-
-		instr += count * size;
-	    } break;
+            case D3DOP_SPAN:
+                WARN("SPAN-s           (%u)\n", count);
+                instr += count * size;
+                break;
 
             case D3DOP_SETSTATUS:
                 TRACE("SETSTATUS        (%d)\n", count);
@@ -410,12 +436,11 @@ HRESULT d3d_execute_buffer_execute(struct d3d_execute_buffer *buffer,
                 }
                 break;
 
-	    default:
-	        ERR("Unhandled OpCode %d !!!\n",current->bOpcode);
-	        /* Try to save ... */
-	        instr += count * size;
-	        break;
-	}
+            default:
+                ERR("Unhandled OpCode %#x.\n",current->bOpcode);
+                instr += count * size;
+                break;
+        }
     }
 
 end_of_buffer:
@@ -483,7 +508,7 @@ static ULONG WINAPI d3d_execute_buffer_Release(IDirect3DExecuteBuffer *iface)
     if (!ref)
     {
         if (buffer->need_free)
-            HeapFree(GetProcessHeap(), 0, buffer->desc.lpData);
+            heap_free(buffer->desc.lpData);
         if (buffer->index_buffer)
             wined3d_buffer_decref(buffer->index_buffer);
         if (buffer->dst_vertex_buffer)
@@ -491,7 +516,7 @@ static ULONG WINAPI d3d_execute_buffer_Release(IDirect3DExecuteBuffer *iface)
             wined3d_buffer_decref(buffer->src_vertex_buffer);
             wined3d_buffer_decref(buffer->dst_vertex_buffer);
         }
-        HeapFree(GetProcessHeap(), 0, buffer);
+        heap_free(buffer);
     }
 
     return ref;
@@ -592,17 +617,25 @@ static HRESULT WINAPI d3d_execute_buffer_SetExecuteData(IDirect3DExecuteBuffer *
     {
         unsigned int new_size = max(data->dwVertexCount, buffer->vertex_size * 2);
         struct wined3d_buffer *src_buffer, *dst_buffer;
+        struct wined3d_buffer_desc desc;
 
-        hr = wined3d_buffer_create_vb(buffer->d3ddev->wined3d_device, new_size * sizeof(D3DVERTEX),
-                WINED3DUSAGE_DYNAMIC | WINED3DUSAGE_WRITEONLY, WINED3D_POOL_SYSTEM_MEM,
-                NULL, &ddraw_null_wined3d_parent_ops, &src_buffer);
-        if (FAILED(hr))
+        desc.byte_width = new_size * sizeof(D3DVERTEX);
+        desc.usage = 0;
+        desc.bind_flags = WINED3D_BIND_VERTEX_BUFFER;
+        desc.access = WINED3D_RESOURCE_ACCESS_CPU | WINED3D_RESOURCE_ACCESS_MAP;
+        desc.misc_flags = 0;
+        desc.structure_byte_stride = 0;
+
+        if (FAILED(hr = wined3d_buffer_create(buffer->d3ddev->wined3d_device, &desc,
+                NULL, NULL, &ddraw_null_wined3d_parent_ops, &src_buffer)))
             return hr;
 
-        hr = wined3d_buffer_create_vb(buffer->d3ddev->wined3d_device, new_size * sizeof(D3DTLVERTEX),
-                WINED3DUSAGE_STATICDECL, WINED3D_POOL_DEFAULT,
-                NULL, &ddraw_null_wined3d_parent_ops, &dst_buffer);
-        if (FAILED(hr))
+        desc.byte_width = new_size * sizeof(D3DTLVERTEX);
+        desc.usage = WINED3DUSAGE_STATICDECL;
+        desc.access = WINED3D_RESOURCE_ACCESS_GPU | WINED3D_RESOURCE_ACCESS_MAP;
+
+        if (FAILED(hr = wined3d_buffer_create(buffer->d3ddev->wined3d_device, &desc,
+                NULL, NULL, &ddraw_null_wined3d_parent_ops, &dst_buffer)))
         {
             wined3d_buffer_decref(src_buffer);
             return hr;
@@ -627,9 +660,8 @@ static HRESULT WINAPI d3d_execute_buffer_SetExecuteData(IDirect3DExecuteBuffer *
     {
         box.left = buffer->src_vertex_pos * sizeof(D3DVERTEX);
         box.right = box.left + data->dwVertexCount * sizeof(D3DVERTEX);
-        hr = wined3d_resource_map(wined3d_buffer_get_resource(buffer->src_vertex_buffer), 0,
-                &map_desc, &box, buffer->src_vertex_pos ? WINED3D_MAP_NOOVERWRITE : WINED3D_MAP_DISCARD);
-        if (FAILED(hr))
+        if (FAILED(hr = wined3d_resource_map(wined3d_buffer_get_resource(buffer->src_vertex_buffer),
+                0, &map_desc, &box, 0)))
             return hr;
 
         memcpy(map_desc.data, ((BYTE *)buffer->desc.lpData) + data->dwVertexOffset,
@@ -759,8 +791,7 @@ HRESULT d3d_execute_buffer_init(struct d3d_execute_buffer *execute_buffer,
     if (!execute_buffer->desc.lpData && execute_buffer->desc.dwBufferSize)
     {
         execute_buffer->need_free = TRUE;
-        execute_buffer->desc.lpData = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, execute_buffer->desc.dwBufferSize);
-        if (!execute_buffer->desc.lpData)
+        if (!(execute_buffer->desc.lpData = heap_alloc_zero(execute_buffer->desc.dwBufferSize)))
         {
             ERR("Failed to allocate execute buffer data.\n");
             return DDERR_OUTOFMEMORY;
