@@ -24,11 +24,6 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(ddraw);
 
-static inline struct d3d_vertex_buffer *impl_from_IDirect3DVertexBuffer(IDirect3DVertexBuffer *iface)
-{
-    return CONTAINING_RECORD(iface, struct d3d_vertex_buffer, IDirect3DVertexBuffer_iface);
-}
-
 static inline struct d3d_vertex_buffer *impl_from_IDirect3DVertexBuffer7(IDirect3DVertexBuffer7 *iface)
 {
     return CONTAINING_RECORD(iface, struct d3d_vertex_buffer, IDirect3DVertexBuffer7_iface);
@@ -38,75 +33,37 @@ static inline struct d3d_vertex_buffer *impl_from_IDirect3DVertexBuffer7(IDirect
  * IUnknown Methods
  *****************************************************************************/
 
-/*****************************************************************************
- * IDirect3DVertexBuffer7::QueryInterface
- *
- * The QueryInterface Method for Vertex Buffers
- * For a link to QueryInterface rules, see IDirectDraw7::QueryInterface
- *
- * Params
- *  riid: Queried Interface id
- *  obj: Address to return the interface pointer
- *
- * Returns:
- *  S_OK on success
- *  E_NOINTERFACE if the interface wasn't found
- *
- *****************************************************************************/
 static HRESULT WINAPI d3d_vertex_buffer7_QueryInterface(IDirect3DVertexBuffer7 *iface, REFIID riid, void  **obj)
 {
     struct d3d_vertex_buffer *buffer = impl_from_IDirect3DVertexBuffer7(iface);
 
     TRACE("iface %p, riid %s, object %p.\n", iface, debugstr_guid(riid), obj);
 
-    /* By default, set the object pointer to NULL */
     *obj = NULL;
 
-    if ( IsEqualGUID( &IID_IUnknown,  riid ) )
+    if (IsEqualGUID(&IID_IUnknown, riid))
     {
         IDirect3DVertexBuffer7_AddRef(iface);
         *obj = iface;
-        TRACE("  Creating IUnknown interface at %p.\n", *obj);
         return S_OK;
     }
-    if ( IsEqualGUID( &IID_IDirect3DVertexBuffer, riid ) )
+    if (IsEqualGUID(&IID_IDirect3DVertexBuffer7, riid) && buffer->version == 7)
     {
         IDirect3DVertexBuffer7_AddRef(iface);
-        *obj = &buffer->IDirect3DVertexBuffer_iface;
-        TRACE("  Creating IDirect3DVertexBuffer interface %p\n", *obj);
+        *obj = iface;
         return S_OK;
     }
-    if ( IsEqualGUID( &IID_IDirect3DVertexBuffer7, riid ) )
+    if (IsEqualGUID(&IID_IDirect3DVertexBuffer, riid) && buffer->version == 3)
     {
         IDirect3DVertexBuffer7_AddRef(iface);
         *obj = iface;
-        TRACE("  Creating IDirect3DVertexBuffer7 interface %p\n", *obj);
         return S_OK;
     }
 
     WARN("%s not implemented, returning E_NOINTERFACE.\n", debugstr_guid(riid));
-
     return E_NOINTERFACE;
 }
 
-static HRESULT WINAPI d3d_vertex_buffer1_QueryInterface(IDirect3DVertexBuffer *iface, REFIID riid, void **obj)
-{
-    struct d3d_vertex_buffer *buffer = impl_from_IDirect3DVertexBuffer(iface);
-
-    TRACE("iface %p, riid %s, object %p.\n", iface, debugstr_guid(riid), obj);
-
-    return d3d_vertex_buffer7_QueryInterface(&buffer->IDirect3DVertexBuffer7_iface, riid, obj);
-}
-
-/*****************************************************************************
- * IDirect3DVertexBuffer7::AddRef
- *
- * AddRef for Vertex Buffers
- *
- * Returns:
- *  The new refcount
- *
- *****************************************************************************/
 static ULONG WINAPI d3d_vertex_buffer7_AddRef(IDirect3DVertexBuffer7 *iface)
 {
     struct d3d_vertex_buffer *buffer = impl_from_IDirect3DVertexBuffer7(iface);
@@ -117,25 +74,6 @@ static ULONG WINAPI d3d_vertex_buffer7_AddRef(IDirect3DVertexBuffer7 *iface)
     return ref;
 }
 
-static ULONG WINAPI d3d_vertex_buffer1_AddRef(IDirect3DVertexBuffer *iface)
-{
-    struct d3d_vertex_buffer *buffer = impl_from_IDirect3DVertexBuffer(iface);
-
-    TRACE("iface %p.\n", iface);
-
-    return d3d_vertex_buffer7_AddRef(&buffer->IDirect3DVertexBuffer7_iface);
-}
-
-
-/*****************************************************************************
- * IDirect3DVertexBuffer7::Release
- *
- * Release for Vertex Buffers
- *
- * Returns:
- *  The new refcount
- *
- *****************************************************************************/
 static ULONG WINAPI d3d_vertex_buffer7_Release(IDirect3DVertexBuffer7 *iface)
 {
     struct d3d_vertex_buffer *buffer = impl_from_IDirect3DVertexBuffer7(iface);
@@ -143,40 +81,33 @@ static ULONG WINAPI d3d_vertex_buffer7_Release(IDirect3DVertexBuffer7 *iface)
 
     TRACE("%p decreasing refcount to %u.\n", buffer, ref);
 
-    if (ref == 0)
+    if (!ref)
     {
-        struct wined3d_buffer *curVB = NULL;
+        struct wined3d_buffer *vb = NULL;
         UINT offset, stride;
 
-        /* D3D7 Vertex buffers don't stay bound in the device, they are passed
-         * as a parameter to drawPrimitiveVB. DrawPrimitiveVB sets them as the
-         * stream source in wined3d, and they should get unset there before
+        /* D3D7 vertex buffers don't stay bound in the device, they are passed
+         * as a parameter to DrawPrimitiveVB. DrawPrimitiveVB sets them as the
+         * stream source in wined3d and they should get unset there before
          * they are destroyed. */
         wined3d_mutex_lock();
-        wined3d_device_get_stream_source(buffer->ddraw->wined3d_device,
-                0, &curVB, &offset, &stride);
-        if (curVB == buffer->wined3d_buffer)
+        wined3d_device_get_stream_source(buffer->ddraw->wined3d_device, 0, &vb, &offset, &stride);
+        if (vb == buffer->wined3d_buffer)
             wined3d_device_set_stream_source(buffer->ddraw->wined3d_device, 0, NULL, 0, 0);
 
         wined3d_vertex_declaration_decref(buffer->wined3d_declaration);
         wined3d_buffer_decref(buffer->wined3d_buffer);
         wined3d_mutex_unlock();
 
-        HeapFree(GetProcessHeap(), 0, buffer);
+        if (buffer->version == 7)
+            IDirectDraw7_Release(&buffer->ddraw->IDirectDraw7_iface);
+
+        heap_free(buffer);
     }
 
     return ref;
 }
 
-static ULONG WINAPI d3d_vertex_buffer1_Release(IDirect3DVertexBuffer *iface)
-{
-    struct d3d_vertex_buffer *buffer = impl_from_IDirect3DVertexBuffer(iface);
-
-    TRACE("iface %p.\n", iface);
-
-    return d3d_vertex_buffer7_Release(&buffer->IDirect3DVertexBuffer7_iface);
-}
-
 /*****************************************************************************
  * IDirect3DVertexBuffer Methods
  *****************************************************************************/
@@ -184,22 +115,24 @@ static ULONG WINAPI d3d_vertex_buffer1_Release(IDirect3DVertexBuffer *iface)
 static HRESULT d3d_vertex_buffer_create_wined3d_buffer(struct d3d_vertex_buffer *buffer, BOOL dynamic,
         struct wined3d_buffer **wined3d_buffer)
 {
-    DWORD usage = WINED3DUSAGE_STATICDECL;
-    enum wined3d_pool pool;
-
-    if (buffer->Caps & D3DVBCAPS_SYSTEMMEMORY)
-        pool = WINED3D_POOL_SYSTEM_MEM;
-    else
-        pool = WINED3D_POOL_DEFAULT;
+    struct wined3d_buffer_desc desc;
 
+    desc.byte_width = buffer->size;
+    desc.usage = WINED3DUSAGE_STATICDECL;
     if (buffer->Caps & D3DVBCAPS_WRITEONLY)
-        usage |= WINED3DUSAGE_WRITEONLY;
+        desc.usage |= WINED3DUSAGE_WRITEONLY;
     if (dynamic)
-        usage |= WINED3DUSAGE_DYNAMIC;
+        desc.usage |= WINED3DUSAGE_DYNAMIC;
+    desc.bind_flags = WINED3D_BIND_VERTEX_BUFFER;
+    if (buffer->Caps & D3DVBCAPS_SYSTEMMEMORY)
+        desc.access = WINED3D_RESOURCE_ACCESS_CPU | WINED3D_RESOURCE_ACCESS_MAP;
+    else
+        desc.access = WINED3D_RESOURCE_ACCESS_GPU | WINED3D_RESOURCE_ACCESS_MAP;
+    desc.misc_flags = 0;
+    desc.structure_byte_stride = 0;
 
-    return wined3d_buffer_create_vb(buffer->ddraw->wined3d_device,
-        buffer->size, usage, pool, buffer, &ddraw_null_wined3d_parent_ops,
-        wined3d_buffer);
+    return wined3d_buffer_create(buffer->ddraw->wined3d_device, &desc,
+            NULL, buffer, &ddraw_null_wined3d_parent_ops, wined3d_buffer);
 }
 
 /*****************************************************************************
@@ -229,23 +162,16 @@ static HRESULT WINAPI d3d_vertex_buffer7_Lock(IDirect3DVertexBuffer7 *iface,
     struct wined3d_resource *wined3d_resource;
     struct wined3d_map_desc wined3d_map_desc;
     HRESULT hr;
-    DWORD wined3d_flags = 0;
 
     TRACE("iface %p, flags %#x, data %p, data_size %p.\n", iface, flags, data, data_size);
 
-    /* Writeonly: Pointless. Event: Unsupported by native according to the sdk
-     * nosyslock: Not applicable
-     */
+    if (buffer->version != 7)
+        flags &= ~(DDLOCK_NOOVERWRITE | DDLOCK_DISCARDCONTENTS);
+
     if (!(flags & DDLOCK_WAIT))
-        wined3d_flags |= WINED3D_MAP_DONOTWAIT;
-    if (flags & DDLOCK_READONLY)
-        wined3d_flags |= WINED3D_MAP_READONLY;
-    if (flags & DDLOCK_NOOVERWRITE)
-        wined3d_flags |= WINED3D_MAP_NOOVERWRITE;
+        flags |= DDLOCK_DONOTWAIT;
     if (flags & DDLOCK_DISCARDCONTENTS)
     {
-        wined3d_flags |= WINED3D_MAP_DISCARD;
-
         if (!buffer->dynamic)
         {
             struct wined3d_buffer *new_buffer;
@@ -275,7 +201,7 @@ static HRESULT WINAPI d3d_vertex_buffer7_Lock(IDirect3DVertexBuffer7 *iface,
     }
 
     hr = wined3d_resource_map(wined3d_buffer_get_resource(buffer->wined3d_buffer),
-            0, &wined3d_map_desc, NULL, wined3d_flags);
+            0, &wined3d_map_desc, NULL, wined3dmapflags_from_ddrawmapflags(flags));
     *data = wined3d_map_desc.data;
 
     wined3d_mutex_unlock();
@@ -283,16 +209,6 @@ static HRESULT WINAPI d3d_vertex_buffer7_Lock(IDirect3DVertexBuffer7 *iface,
     return hr;
 }
 
-static HRESULT WINAPI d3d_vertex_buffer1_Lock(IDirect3DVertexBuffer *iface,
-        DWORD flags, void **data, DWORD *data_size)
-{
-    struct d3d_vertex_buffer *buffer = impl_from_IDirect3DVertexBuffer(iface);
-
-    TRACE("iface %p, flags %#x, data %p, data_size %p.\n", iface, flags, data, data_size);
-
-    return d3d_vertex_buffer7_Lock(&buffer->IDirect3DVertexBuffer7_iface, flags, data, data_size);
-}
-
 /*****************************************************************************
  * IDirect3DVertexBuffer7::Unlock
  *
@@ -315,16 +231,6 @@ static HRESULT WINAPI d3d_vertex_buffer7_Unlock(IDirect3DVertexBuffer7 *iface)
     return D3D_OK;
 }
 
-static HRESULT WINAPI d3d_vertex_buffer1_Unlock(IDirect3DVertexBuffer *iface)
-{
-    struct d3d_vertex_buffer *buffer = impl_from_IDirect3DVertexBuffer(iface);
-
-    TRACE("iface %p.\n", iface);
-
-    return d3d_vertex_buffer7_Unlock(&buffer->IDirect3DVertexBuffer7_iface);
-}
-
-
 /*****************************************************************************
  * IDirect3DVertexBuffer7::ProcessVertices
  *
@@ -340,7 +246,7 @@ static HRESULT WINAPI d3d_vertex_buffer1_Unlock(IDirect3DVertexBuffer *iface)
  *  SrcIndex: Index of the first vertex in the src buffer to process
  *  D3DDevice: Device to use for transformation
  *  Flags: 0 for default, D3DPV_DONOTCOPYDATA to prevent copying
- *         unchaned vertices
+ *         unchanged vertices
  *
  * Returns:
  *  D3D_OK on success
@@ -353,7 +259,9 @@ static HRESULT WINAPI d3d_vertex_buffer7_ProcessVertices(IDirect3DVertexBuffer7
 {
     struct d3d_vertex_buffer *dst_buffer_impl = impl_from_IDirect3DVertexBuffer7(iface);
     struct d3d_vertex_buffer *src_buffer_impl = unsafe_impl_from_IDirect3DVertexBuffer7(src_buffer);
-    struct d3d_device *device_impl = unsafe_impl_from_IDirect3DDevice7(device);
+    struct d3d_device *device_impl = dst_buffer_impl->version == 7
+            ? unsafe_impl_from_IDirect3DDevice7(device)
+            : unsafe_impl_from_IDirect3DDevice3((IDirect3DDevice3 *)device);
     BOOL oldClip, doClip;
     HRESULT hr;
 
@@ -399,22 +307,6 @@ static HRESULT WINAPI d3d_vertex_buffer7_ProcessVertices(IDirect3DVertexBuffer7
     return hr;
 }
 
-static HRESULT WINAPI d3d_vertex_buffer1_ProcessVertices(IDirect3DVertexBuffer *iface,
-        DWORD vertex_op, DWORD dst_idx, DWORD count, IDirect3DVertexBuffer *src_buffer,
-        DWORD src_idx, IDirect3DDevice3 *device, DWORD flags)
-{
-    struct d3d_vertex_buffer *dst_buffer_impl = impl_from_IDirect3DVertexBuffer(iface);
-    struct d3d_vertex_buffer *src_buffer_impl = unsafe_impl_from_IDirect3DVertexBuffer(src_buffer);
-    struct d3d_device *device_impl = unsafe_impl_from_IDirect3DDevice3(device);
-
-    TRACE("iface %p, vertex_op %#x, dst_idx %u, count %u, src_buffer %p, src_idx %u, device %p, flags %#x.\n",
-            iface, vertex_op, dst_idx, count, src_buffer, src_idx, device, flags);
-
-    return d3d_vertex_buffer7_ProcessVertices(&dst_buffer_impl->IDirect3DVertexBuffer7_iface, vertex_op,
-            dst_idx, count, &src_buffer_impl->IDirect3DVertexBuffer7_iface, src_idx,
-            device_impl ? &device_impl->IDirect3DDevice7_iface : NULL, flags);
-}
-
 /*****************************************************************************
  * IDirect3DVertexBuffer7::GetVertexBufferDesc
  *
@@ -451,16 +343,6 @@ static HRESULT WINAPI d3d_vertex_buffer7_GetVertexBufferDesc(IDirect3DVertexBuff
     return D3D_OK;
 }
 
-static HRESULT WINAPI d3d_vertex_buffer1_GetVertexBufferDesc(IDirect3DVertexBuffer *iface, D3DVERTEXBUFFERDESC *desc)
-{
-    struct d3d_vertex_buffer *buffer = impl_from_IDirect3DVertexBuffer(iface);
-
-    TRACE("iface %p, desc %p.\n", iface, desc);
-
-    return d3d_vertex_buffer7_GetVertexBufferDesc(&buffer->IDirect3DVertexBuffer7_iface, desc);
-}
-
-
 /*****************************************************************************
  * IDirect3DVertexBuffer7::Optimize
  *
@@ -498,18 +380,6 @@ static HRESULT WINAPI d3d_vertex_buffer7_Optimize(IDirect3DVertexBuffer7 *iface,
     return DD_OK;
 }
 
-static HRESULT WINAPI d3d_vertex_buffer1_Optimize(IDirect3DVertexBuffer *iface,
-        IDirect3DDevice3 *device, DWORD flags)
-{
-    struct d3d_vertex_buffer *buffer = impl_from_IDirect3DVertexBuffer(iface);
-    struct d3d_device *device_impl = unsafe_impl_from_IDirect3DDevice3(device);
-
-    TRACE("iface %p, device %p, flags %#x.\n", iface, device, flags);
-
-    return d3d_vertex_buffer7_Optimize(&buffer->IDirect3DVertexBuffer7_iface,
-            device_impl ? &device_impl->IDirect3DDevice7_iface : NULL, flags);
-}
-
 /*****************************************************************************
  * IDirect3DVertexBuffer7::ProcessVerticesStrided
  *
@@ -559,18 +429,6 @@ static const struct IDirect3DVertexBuffer7Vtbl d3d_vertex_buffer7_vtbl =
     d3d_vertex_buffer7_ProcessVerticesStrided,
 };
 
-static const struct IDirect3DVertexBufferVtbl d3d_vertex_buffer1_vtbl =
-{
-    d3d_vertex_buffer1_QueryInterface,
-    d3d_vertex_buffer1_AddRef,
-    d3d_vertex_buffer1_Release,
-    d3d_vertex_buffer1_Lock,
-    d3d_vertex_buffer1_Unlock,
-    d3d_vertex_buffer1_ProcessVertices,
-    d3d_vertex_buffer1_GetVertexBufferDesc,
-    d3d_vertex_buffer1_Optimize,
-};
-
 HRESULT d3d_vertex_buffer_create(struct d3d_vertex_buffer **vertex_buf,
         struct ddraw *ddraw, D3DVERTEXBUFFERDESC *desc)
 {
@@ -583,14 +441,14 @@ HRESULT d3d_vertex_buffer_create(struct d3d_vertex_buffer **vertex_buf,
     TRACE("    FVF %#x\n", desc->dwFVF);
     TRACE("    dwNumVertices %u\n", desc->dwNumVertices);
 
-    buffer = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*buffer));
-    if (!buffer)
+    if (!(buffer = heap_alloc_zero(sizeof(*buffer))))
         return DDERR_OUTOFMEMORY;
 
     buffer->IDirect3DVertexBuffer7_iface.lpVtbl = &d3d_vertex_buffer7_vtbl;
-    buffer->IDirect3DVertexBuffer_iface.lpVtbl = &d3d_vertex_buffer1_vtbl;
     buffer->ref = 1;
-
+    buffer->version = ddraw->d3dversion;
+    if (buffer->version == 7)
+        IDirectDraw7_AddRef(&ddraw->IDirectDraw7_iface);
     buffer->ddraw = ddraw;
     buffer->Caps = desc->dwCaps;
     buffer->fvf = desc->dwFVF;
@@ -620,20 +478,11 @@ end:
     if (hr == D3D_OK)
         *vertex_buf = buffer;
     else
-        HeapFree(GetProcessHeap(), 0, buffer);
+        heap_free(buffer);
 
     return hr;
 }
 
-struct d3d_vertex_buffer *unsafe_impl_from_IDirect3DVertexBuffer(IDirect3DVertexBuffer *iface)
-{
-    if (!iface)
-        return NULL;
-    assert(iface->lpVtbl == &d3d_vertex_buffer1_vtbl);
-
-    return impl_from_IDirect3DVertexBuffer(iface);
-}
-
 struct d3d_vertex_buffer *unsafe_impl_from_IDirect3DVertexBuffer7(IDirect3DVertexBuffer7 *iface)
 {
     if (!iface)
